@@ -14,34 +14,34 @@ CompassController::~CompassController() = default;
 
 auto CompassController::getType() const -> GeometryToolType { return GeometryToolType::COMPASS; }
 
-auto CompassController::posRelToSide(double x, double y) const -> utl::Point<double> {
+auto CompassController::posRelToSide(double x, double y) const -> xoj::util::Point<double> {
     cairo_matrix_t inv = geometryTool->getMatrix();
     cairo_matrix_invert(&inv);
     cairo_matrix_transform_point(&inv, &x, &y);
-    return utl::Point<double>(x, -y);
+    return xoj::util::Point<double>(x, -y);
 }
 
 auto CompassController::isInsideGeometryTool(double x, double y, double border) const -> bool {
-    const utl::Point<double> p = posRelToSide(x, y);
+    const xoj::util::Point<double> p = posRelToSide(x, y);
     return std::hypot(p.x, p.y) <= geometryTool->getHeight() + border;
 }
 
-auto CompassController::getPointForAngle(double a) const -> utl::Point<double> {
+auto CompassController::getPointForAngle(double a) const -> xoj::util::Point<double> {
     cairo_matrix_t matrix = geometryTool->getMatrix();
     double x = geometryTool->getHeight() * std::cos(a);
     double y = geometryTool->getHeight() * std::sin(a);
     cairo_matrix_transform_point(&matrix, &x, &y);
 
-    return utl::Point<double>(x, y);
+    return xoj::util::Point<double>(x, y);
 }
 
-auto CompassController::getPointForRadius(double r) const -> utl::Point<double> {
+auto CompassController::getPointForRadius(double r) const -> xoj::util::Point<double> {
     cairo_matrix_t matrix = geometryTool->getMatrix();
     double x = r;
     double y = 0.;
     cairo_matrix_transform_point(&matrix, &x, &y);
 
-    return utl::Point<double>(x, y);
+    return xoj::util::Point<double>(x, y);
 }
 
 void CompassController::createOutlineStroke(double a) {
@@ -49,7 +49,7 @@ void CompassController::createOutlineStroke(double a) {
         angleMax = a;
         angleMin = a;
 
-        const utl::Point<double> p = this->getPointForAngle(a);
+        const xoj::util::Point<double> p = this->getPointForAngle(a);
         initializeStroke();
         stroke->addPoint(Point(p.x, p.y));
         stroke->addPoint(Point(p.x, p.y));  // doubled point
@@ -64,7 +64,7 @@ void CompassController::createRadialStroke(double x) {
         radiusMax = x;
         radiusMin = x;
 
-        const utl::Point<double> p = posRelToSide(x, 0.);
+        const xoj::util::Point<double> p = posRelToSide(x, 0.);
         initializeStroke();
         stroke->addPoint(Point(p.x, p.y));
         stroke->addPoint(Point(p.x, p.y));  // doubled point
@@ -80,13 +80,13 @@ void CompassController::updateOutlineStroke(double x) {
     stroke->deletePointsFrom(0);
     const auto h = view->getXournal()->getControl()->getToolHandler();
     const bool filled = (h->getFill() != -1);
-    const utl::Point<double> c = utl::Point<double>{geometryTool->getTranslationX(), geometryTool->getTranslationY()};
+    const xoj::util::Point<double>& c = this->getGeometryTool()->getOrigin();
 
     if (filled && angleMax < angleMin + 2 * M_PI) {
         stroke->addPoint(Point(c.x, c.y));
     }
     for (auto i = 0; i <= 100; i++) {
-        const utl::Point<double> p =
+        const xoj::util::Point<double> p =
                 getPointForAngle(angleMin + static_cast<double>(i) / 100.0 * std::min(angleMax - angleMin, 2 * M_PI));
         stroke->addPoint(Point(p.x, p.y));
     }
@@ -100,8 +100,8 @@ void CompassController::updateRadialStroke(double x) {
     radiusMax = std::max(this->radiusMax, x);
     radiusMin = std::min(this->radiusMin, x);
     stroke->deletePointsFrom(0);
-    const utl::Point<double> p1 = getPointForRadius(radiusMin);
-    const utl::Point<double> p2 = getPointForRadius(radiusMax);
+    const xoj::util::Point<double> p1 = getPointForRadius(radiusMin);
+    const xoj::util::Point<double> p2 = getPointForRadius(radiusMax);
 
     stroke->addPoint(Point(p1.x, p1.y));
     stroke->addPoint(Point(p2.x, p2.y));
