@@ -11,28 +11,29 @@
 
 #pragma once
 
-#include <cstddef>  // for size_t
-#include <memory>   // for unique_ptr
-#include <string>   // for string, allocator
-#include <vector>   // for vector
+#include <cstddef>   // for size_t
+#include <memory>    // for unique_ptr
+#include <optional>  // for optional
+#include <string>    // for string, allocator
+#include <vector>    // for vector
 
 #include <gdk-pixbuf/gdk-pixbuf.h>  // for GdkPixbuf
 #include <gio/gio.h>                // for GApplication
 #include <glib.h>                   // for guint
 #include <gtk/gtk.h>                // for GtkLabel
 
-#include "control/ToolEnums.h"              // for ToolSize, ToolType
-#include "control/jobs/ProgressListener.h"  // for ProgressListener
-#include "control/settings/ViewModes.h"     // for ViewModeId
-#include "enums/ActionGroup.enum.h"         // for ActionGroup
-#include "enums/ActionType.enum.h"          // for ActionType
-#include "model/DocumentHandler.h"          // for DocumentHandler
-#include "model/DocumentListener.h"         // for DocumentListener
-#include "model/GeometryTool.h"             // for GeometryTool
-#include "model/PageRef.h"                  // for PageRef
-#include "undo/UndoRedoHandler.h"           // for UndoRedoHandler (ptr only)
+#include "control/ToolEnums.h"                      // for ToolSize, ToolType
+#include "control/jobs/ProgressListener.h"          // for ProgressListener
+#include "control/settings/ViewModes.h"             // for ViewModeId
+#include "control/tools/EditSelection.h"            // for OrderChange
+#include "enums/Action.enum.h"                      // for Action
+#include "gui/toolbarMenubar/model/ColorPalette.h"  // for ColorPalette
+#include "model/DocumentHandler.h"                  // for DocumentHandler
+#include "model/DocumentListener.h"                 // for DocumentListener
+#include "model/GeometryTool.h"                     // for GeometryTool
+#include "model/PageRef.h"                          // for PageRef
+#include "undo/UndoRedoHandler.h"                   // for UndoRedoHandler (ptr only)
 
-#include "Actions.h"           // for ActionHandler
 #include "ClipboardHandler.h"  // for ClipboardListener
 #include "ToolHandler.h"       // for ToolListener
 #include "filesystem.h"        // for path
@@ -50,7 +51,6 @@ class MetadataEntry;
 class MetadataCallbackData;
 class PageBackgroundChangeController;
 class PageTypeHandler;
-class PageTypeMenu;
 class BaseExportJob;
 class LayerController;
 class PluginController;
@@ -67,9 +67,11 @@ class XournalScheduler;
 class ZoomControl;
 class ToolMenuHandler;
 class XojFont;
+class XojPdfRectangle;
+class Callback;
+class ActionDatabase;
 
 class Control:
-        public ActionHandler,
         public ToolListener,
         public DocumentHandler,
         public UndoRedoListener,
@@ -86,23 +88,44 @@ public:
     void initWindow(MainWindow* win);
 
 public:
-    // Menu File
-    bool newFile(std::string pageTemplate = "", fs::path filepath = {});
-    bool openFile(fs::path filepath = "", int scrollToPage = -1, bool forceOpen = false);
-    bool annotatePdf(fs::path filepath, bool attachPdf, bool attachToDocument);
+    /// Asymchronously closes the current document and replaces it by a new file
+    void newFile(fs::path filepath = {});
+
+    /// @brief Shows an open file dialog and opens the selected file after closing the previously opened file
+    void askToOpenFile();
+    /**
+     * @brief Asynchronously opens the provided path, safely closes the current opened document and replaces it with the
+     * newly parsed file. Calls callback afterwards, with boolean parameter true on success. Does nothing to this->doc
+     * in case of failure at any point.
+     */
+    void openFile(
+            fs::path filepath, std::function<void(bool)> callback = [](bool) {}, int scrollToPage = -1,
+            bool forceOpen = false);
+    /// Shows an open file dialog and opens the selected file
+    void askToAnnotatePdf();
+
+    /**
+     * (Potentially asynchronously) Opens the given file without saving any previously opened Document. Calls callback
+     * afterwards, with boolean parameter true if success. WARNING: may lead to data loss if the current Document has
+     * not been saved yet.
+     */
+    void openFileWithoutSavingTheCurrentDocument(fs::path filepath, bool attachToDocument, int scrollToPage,
+                                                 std::function<void(bool)> callback);
+
     void print();
     void exportAsPdf();
     void exportAs();
-    void exportBase(BaseExportJob* job);
     void quit(bool allowCancel = true);
 
     /**
-     * Save the current document.
-     *
-     * @param synchron Whether the save should be run synchronously or asynchronously.
+     * @brief Asynchronously saves the document and calls callback afterwards with boolean parameter true on success.
+     * May ask the user for a place to save if necessary.
      */
-    bool save(bool synchron = false);
-    bool saveAs();
+    void save(std::function<void(bool)> callback = [](bool) {});
+    /**
+     * @brief Asks the user for a new location, asynchronously saves the document there and calls callback afterwards.
+     */
+    void saveAs(std::function<void(bool)> callback = [](bool) {});
 
     /**
      * Marks the current document as saved if it is currently marked as unsaved.
@@ -112,18 +135,12 @@ public:
     /**
      * Close the current document, prompting to save unsaved changes.
      *
+     * @param callback Called after trying to close the document, with param true in case of success, false otherwise.
      * @param allowDestroy Whether clicking "Discard" should destroy the current document.
      * @param allowCancel Whether the user should be able to cancel closing the document.
      * @return true if the user closed the document, otherwise false.
      */
-    bool close(bool allowDestroy = false, bool allowCancel = true);
-
-    /**
-     * @brief Asks user to replace an existing file when saving / exporting, since we add the extension
-     *        after the OK, we need to check manually
-     * @param parent the dialog which wants to override something
-     */
-    bool askToReplace(fs::path const& filepath, GtkWindow* parent) const;
+    void close(std::function<void(bool)> callback, bool allowDestroy = false, bool allowCancel = true);
 
     // Menu edit
     void showSettings();
@@ -133,8 +150,7 @@ public:
 
     // Menu Help
     void showAbout();
-
-    void actionPerformed(ActionType type, ActionGroup group, GtkToolButton* toolbutton, bool enabled) override;
+    void showGtkDemo();
 
     /**
      * @brief Update the Cursor and the Toolbar based on the active color
@@ -146,7 +162,6 @@ public:
      *
      */
     void changeColorOfSelection() override;
-    void setCustomColorSelected() override;
     void toolChanged() override;
     void toolSizeChanged() override;
     void toolFillChanged() override;
@@ -155,7 +170,8 @@ public:
     void selectTool(ToolType type);
     void selectDefaultTool();
 
-    void setFontSelected(const XojFont& font);
+    void setFontSelected(const XojFont& font);  ///< Modifies the Action state without triggering callbacks
+    void fontChanged(const XojFont& font);      ///< Set the font after the user selected a font
 
     void updatePageNumbers(size_t page, size_t pdfPage);
 
@@ -178,10 +194,13 @@ public:
     void manageToolbars();
     void customizeToolbars();
     void setFullscreen(bool enabled);
+    void setShowSidebar(bool enabled);
+    void setShowToolbar(bool enabled);
+    void setShowMenubar(bool enabled);
 
     void gotoPage();
 
-    void setShapeTool(ActionType type, bool enabled);
+    void setToolDrawingType(DrawingType type);
 
     void paperTemplate();
     void paperFormat();
@@ -193,16 +212,15 @@ public:
      */
     bool loadViewMode(ViewModeId mode);
 
-    bool isFullscreen();
-
     /**
      * @brief Search text on the given page. The matches (if any) are stored in the XojPageView::SearchControl instance.
      * @param occurrences If not nullptr, the pointed variable will contain the number of matches on the page
-     * @param yOfUpperMostMatch If not nullptr, will contain the y coordinate of the first match on the page
+     * @param matchRect If not nullptr, will contain the topleft point of the first match on the page
      *                          (Used for scrolling to the first match)
      * @return true if at least one match was found
      */
-    bool searchTextOnPage(const std::string& text, size_t pageNumber, size_t* occurrences, double* yOfUpperMostMatch);
+    bool searchTextOnPage(const std::string& text, size_t pageNumber, size_t index, size_t* occurrences,
+                          XojPdfRectangle* matchRect);
 
     /**
      * Fire page selected, but first check if the page Number is valid
@@ -212,17 +230,25 @@ public:
     size_t firePageSelected(const PageRef& page);
     void firePageSelected(size_t page);
 
-    void addDefaultPage(std::string pageTemplate);
+    void addDefaultPage(const std::optional<std::string>& pageTemplate, Document* doc = nullptr);
     void duplicatePage();
     void insertNewPage(size_t position, bool shouldScrollToPage = true);
     void appendNewPdfPages();
     void insertPage(const PageRef& page, size_t position, bool shouldScrollToPage = true);
     void deletePage();
+    void movePageTowardsBeginning();
+    void movePageTowardsEnd();
 
     /**
-     * Disable / enable delete page button
+     * Ask the user whether a page with the given id
+     * should be added to the document.
      */
-    void updateDeletePageButton();
+    void askInsertPdfPage(size_t pdfPage);
+
+    /**
+     * Disable / enable page action buttons
+     */
+    void updatePageActions();
 
     // selection handling
     void clearSelection();
@@ -237,12 +263,21 @@ public:
 
     void selectAllOnPage();
 
-    void reorderSelection(ActionType type);
+    void reorderSelection(EditSelection::OrderChange change);
 
     void setToolSize(ToolSize size);
 
+    /**
+     * Change the line style of the PEN if select, or of selected elements if any
+     * Otherwise, select the PEN tool and set its linestyle
+     */
     void setLineStyle(const std::string& style);
 
+    /**
+     * Change the eraser type. If the eraser is not selected, select it as well.
+     */
+    void setEraserType(EraserType type);
+
     void setFill(bool fill);
 
     TextEditor* getTextEditor();
@@ -279,11 +314,17 @@ public:
     SearchBar* getSearchBar() const;
     AudioController* getAudioController() const;
     PageTypeHandler* getPageTypes() const;
-    PageTypeMenu* getNewPageType() const;
     PageBackgroundChangeController* getPageBackgroundChangeController() const;
     LayerController* getLayerController() const;
     PluginController* getPluginController() const;
+    const Palette& getPalette() const;
 
+    /**
+     * Show floating toolbox at specified coordinates
+     * @param x x coordinate relative to main window
+     * @param y y coordinate relative to main window
+     */
+    void showFloatingToolbox(int x, int y);
 
     bool copy();
     bool cut();
@@ -291,7 +332,7 @@ public:
 
     void help();
 
-    void selectFillAlpha(bool pen);
+    void selectAlpha(OpacityFeature feature);
 
     /**
      * @brief Initialize the all button tools based on the respective ButtonConfigs
@@ -307,8 +348,8 @@ public:
 
 public:
     // ProgressListener interface
-    void setMaximumState(int max) override;
-    void setCurrentState(int state) override;
+    void setMaximumState(size_t max) override;
+    void setCurrentState(size_t state) override;
 
 public:
     // ClipboardListener interface
@@ -319,24 +360,19 @@ public:
     void clipboardPasteXournal(ObjectInputStream& in) override;
     void deleteSelection() override;
 
-    void clipboardPaste(Element* e);
+    void clipboardPaste(ElementPtr e);
 
 public:
     void registerPluginToolButtons(ToolMenuHandler* toolMenuHandler);
+    inline ActionDatabase* getActionDatabase() const { return actionDB.get(); }
+    void loadPaletteFromSettings();
 
 protected:
-    /**
-     * This callback is used by used to be called later in the UI Thread
-     * On slower machine this feels more fluent, therefore this will not
-     * be removed
-     */
-    void zoomCallback(ActionType type, bool enabled);
+    void setRotationSnapping(bool enable);
+    void setGridSnapping(bool enable);
 
-    void rotationSnappingToggle();
-    void gridSnappingToggle();
-    void highlightPositionToggle();
-
-    bool showSaveDialog();
+    void showFontDialog();
+    void showColorChooserDialog();
 
     void fileLoaded(int scrollToPage = -1);
 
@@ -347,7 +383,6 @@ protected:
     static bool checkChangedDocument(Control* control);
     static bool autosaveCallback(Control* control);
 
-    void fontChanged();
     /**
      * Load metadata later, md will be deleted
      */
@@ -355,23 +390,39 @@ protected:
 
     static bool loadMetadataCallback(MetadataCallbackData* data);
 
-    /**
-     * Check if this is an autosave file, return false in this case and display a user instruction
-     */
-    bool shouldFileOpen(fs::path const& filepath) const;
-
-    bool loadXoptTemplate(fs::path const& filepath);
-    bool loadPdf(fs::path const& filepath, int scrollToPage);
+    void saveImpl(bool saveAs, std::function<void(bool)> callback);
 
 private:
-    template <class ToolClass, class ViewClass, class ControllerClass, class InputHandlerClass, ActionType a>
-    void makeGeometryTool();
+    /**
+     * @brief Creates the specified geometric tool if it's not on the current page yet. Deletes it if it already exists.
+     * @return true if a geometric tool was created
+     */
+    template <class ToolClass, class ViewClass, class ControllerClass, class InputHandlerClass,
+              GeometryToolType toolType>
+    bool toggleGeometryTool();
     void resetGeometryTool();
 
+    /**
+     * @brief Creates a compass if it's not on the current page yet. Deletes it if it already exists.
+     * @return true if a compass was created
+     */
+    bool toggleCompass();
+    /**
+     * @brief Creates a setsquare if it's not on the current page yet. Deletes it if it already exists.
+     * @return true if a setsquare was created
+     */
+    bool toggleSetsquare();
+
+    struct MissingPdfData;
     /**
      * Prompt the user that the PDF background is missing and offer solution options
      */
-    void promptMissingPdf(LoadHandler& loadHandler, const fs::path& filepath);
+    void promptMissingPdf(MissingPdfData& missingPdf, const fs::path& filepath);
+
+    /**
+     * Handle the response from the missing PDF dialog
+     */
+    void missingPdfDialogResponseHandler(fs::path proposedPdfFilepath, int responseId);
 
     /**
      * "Closes" the document, preparing the editor for a new document.
@@ -379,9 +430,45 @@ private:
     void closeDocument();
 
     /**
-     * Applies the preferred language to the UI
+     * Forcibly replaces the opened document.
+     * WARNING: Be sure the active document has been saved (or discarded) before calling replaceDocument()
      */
-    void applyPreferredLanguage();
+    void replaceDocument(std::unique_ptr<Document> doc, int scrollToPage);
+
+    /**
+     * Asynchronously opens the provided path and parse it as a .xopp or .xoj file. Then forcibly replaces the currently
+     * opened document with the new one. Asks the user in case of doubts (e.g. wrong file version) and aborts if the
+     * document was not correctly and entirely parsed. Calls callback afterwards, with boolean parameter true on
+     * success. WARNING: Be sure the active document has been saved (or discarded) before calling openXoppFile()
+     */
+    void openXoppFile(fs::path filepath, int scrollToPage, std::function<void(bool)> callback);
+
+    /**
+     * Opens the provided path and parse it as a PDF file. Then forcibly replaces the currently opened document with a
+     * new one based on the PDF. WARNING: Be sure the active document has been saved (or discarded) before calling
+     * openPdfFile()
+     *
+     * @return true on success
+     */
+    bool openPdfFile(fs::path filepath, bool attachToDocument, int scrollToPage);
+
+    /**
+     * Opens the provided path and parse it as a PNG file. Then forcibly replaces the currently opened document with a
+     * new one based on the PNG. WARNING: Be sure the active document has been saved (or discarded) before calling
+     * openPngFile()
+     *
+     * @return true on success
+     */
+    bool openPngFile(fs::path filepath, bool attachToDocument, int scrollToPage);
+
+    /**
+     * Opens the provided path and parse it as a .xopt template  file. Then forcibly replaces the currently opened
+     * document with a new one based on the template. WARNING: Be sure the active document has been saved (or discarded)
+     * before calling openXoptFile()
+     *
+     * @return true on success
+     */
+    bool openXoptFile(fs::path filepath);
 
     /**
      * @brief Get the pen line style to select in the toolbar
@@ -395,6 +482,7 @@ private:
     ZoomControl* zoom = nullptr;
 
     Settings* settings = nullptr;
+    std::unique_ptr<Palette> palette;
     MainWindow* win = nullptr;
 
     Document* doc = nullptr;
@@ -404,13 +492,9 @@ private:
 
     ToolHandler* toolHandler;
 
-    ActionType lastAction;
-    ActionGroup lastGroup;
-    bool lastEnabled;
-
     ScrollHandler* scrollHandler;
 
-    AudioController* audioController;
+    std::unique_ptr<AudioController> audioController;
 
     ToolbarDragDropHandler* dragDropHandler = nullptr;
 
@@ -455,7 +539,7 @@ private:
     GtkWidget* statusbar = nullptr;
     GtkLabel* lbState = nullptr;
     GtkProgressBar* pgState = nullptr;
-    int maxState = 0;
+    size_t maxState = 0;
     bool isBlocking;
 
     GladeSearchpath* gladeSearchPath;
@@ -463,9 +547,8 @@ private:
     MetadataManager* metadata;
 
     PageTypeHandler* pageTypes;
-    std::unique_ptr<PageTypeMenu> newPageType;
 
-    PageBackgroundChangeController* pageBackgroundChangeController;
+    std::unique_ptr<PageBackgroundChangeController> pageBackgroundChangeController;
 
     LayerController* layerController;
 
@@ -477,8 +560,7 @@ private:
      */
     PluginController* pluginController;
 
-    /**
-     * Fullscreen handler
-     */
-    FullscreenHandler* fullscreenHandler;
+    std::unique_ptr<ActionDatabase> actionDB;
+    template <Action a>
+    friend struct ActionProperties;
 };
