@@ -27,8 +27,6 @@ public:
     void run() override;
 
 protected:
-    void addFilterToDialog() override;
-    bool testAndSetFilepath(const fs::path& file) override;
-
-private:
+    void addFilterToDialog(GtkFileChooser* dialog) override;
+    void setExtensionFromFilter(fs::path& p, const char* filterName) const override;
 };
