@@ -12,6 +12,7 @@
 #include "model/Point.h"              // for Point, Point::NO_PRESSURE
 #include "model/Stroke.h"             // for Stroke, StrokeTool::ERASER, STR...
 #include "util/Color.h"               // for Color
+#include "util/safe_casts.h"          // for as_unsigned
 
 #include "filesystem.h"  // for path
 
@@ -33,18 +34,24 @@ auto InputHandler::createStroke(Control* control) -> std::unique_ptr<Stroke> {
     if (h->getToolType() == TOOL_PEN) {
         s->setToolType(StrokeTool::PEN);
 
+#ifdef ENABLE_AUDIO
         if (auto* audioController = control->getAudioController(); audioController && audioController->isRecording()) {
             fs::path audioFilename = audioController->getAudioFilename();
             size_t sttime = audioController->getStartTime();
-            size_t milliseconds = ((g_get_monotonic_time() / 1000) - sttime);
+            size_t milliseconds = (as_unsigned(g_get_monotonic_time() / 1000) - sttime);
             s->setTimestamp(milliseconds);
             s->setAudioFilename(audioFilename);
         }
+#endif
     } else if (h->getToolType() == TOOL_HIGHLIGHTER) {
         s->setToolType(StrokeTool::HIGHLIGHTER);
     } else if (h->getToolType() == TOOL_ERASER) {
         s->setToolType(StrokeTool::ERASER);
         s->setColor(Colors::white);
+    } else if (h->getToolType() == TOOL_LASER_POINTER_PEN) {
+        s->setToolType(StrokeTool::PEN);
+    } else if (h->getToolType() == TOOL_LASER_POINTER_HIGHLIGHTER) {
+        s->setToolType(StrokeTool::HIGHLIGHTER);
     }
 
     return s;
