@@ -1,24 +1,22 @@
 #include "VerticalToolHandler.h"
 
 #include <algorithm>  // for max, min, minmax
-#include <cassert>    // for assert
-#include <cmath>      // for abs
 #include <memory>     // for __shared_ptr_access
 
 #include <cairo.h>           // for cairo_fill, cairo_...
 #include <gdk/gdkkeysyms.h>  // for GDK_KEY_Control_L
-#include <glib.h>            // for g_assert, g_assert...
 
+#include "control/Control.h"                       // for Control
 #include "control/tools/SnapToGridInputHandler.h"  // for SnapToGridInputHan...
 #include "gui/LegacyRedrawable.h"                  // for Redrawable
+#include "gui/inputdevices/InputEvents.h"          // for KeyEvent
+#include "model/Document.h"                        // for Document
 #include "model/Element.h"                         // for Element
 #include "model/Layer.h"                           // for Layer
 #include "model/XojPage.h"                         // for XojPage
 #include "undo/MoveUndoAction.h"                   // for MoveUndoAction
 #include "util/DispatchPool.h"
 #include "view/overlays/VerticalToolView.h"
-#include "control/Control.h"  // for Control
-#include "model/Document.h"  // for Document
 
 VerticalToolHandler::VerticalToolHandler(const PageRef& page, Control* control, double y, bool initiallyReverse):
         page(page),
@@ -27,6 +25,7 @@ VerticalToolHandler::VerticalToolHandler(const PageRef& page, Control* control,
         spacingSide(initiallyReverse ? Side::Above : Side::Below),
         snappingHandler(control->getSettings()),
         viewPool(std::make_shared<xoj::util::DispatchPool<xoj::view::VerticalToolView>>()) {
+    snappingHandler.setPageRef(page);
     double ySnapped = snappingHandler.snapVertically(y, false);
     this->startY = ySnapped;
     this->endY = ySnapped;
@@ -42,18 +41,18 @@ void VerticalToolHandler::adoptElements(const Side side) {
     auto* doc = this->control->getDocument();
     doc->lock();
     // Return current elements back to page
-    for (Element* e: this->elements) { this->layer->addElement(e); }
+    for (auto&& e: this->elements) {
+        this->layer->addElement(std::move(e));
+    }
     this->elements.clear();
 
     // Add new elements based on position
-    for (Element* e: this->layer->getElements()) {
+    for (const Element* e: this->layer->getElementsView().clone()) {
         if ((side == Side::Below && e->getY() >= this->startY) ||
             (side == Side::Above && e->getY() + e->getElementHeight() <= this->startY)) {
-            this->elements.push_back(e);
+            this->elements.push_back(this->layer->removeElement(e).e);
         }
     }
-
-    for (Element* e: this->elements) { this->layer->removeElement(e, false); }
     doc->unlock();
 
     Range rg = this->ownedElementsOriginalBoundingBox;
@@ -73,9 +72,8 @@ void VerticalToolHandler::currentPos(double x, double y) {
     this->viewPool->dispatch(xoj::view::VerticalToolView::SET_VERTICAL_SHIFT_REQUEST, ySnapped);
 }
 
-bool VerticalToolHandler::onKeyPressEvent(GdkEventKey* event) {
-    if ((event->keyval == GDK_KEY_Control_L || event->keyval == GDK_KEY_Control_R) &&
-        this->spacingSide == Side::Below) {
+bool VerticalToolHandler::onKeyPressEvent(const KeyEvent& event) {
+    if ((event.keyval == GDK_KEY_Control_L || event.keyval == GDK_KEY_Control_R) && this->spacingSide == Side::Below) {
         this->adoptElements(Side::Above);
         this->viewPool->dispatch(xoj::view::VerticalToolView::SWITCH_DIRECTION_REQUEST);
         return true;
@@ -83,9 +81,8 @@ bool VerticalToolHandler::onKeyPressEvent(GdkEventKey* event) {
     return false;
 }
 
-bool VerticalToolHandler::onKeyReleaseEvent(GdkEventKey* event) {
-    if ((event->keyval == GDK_KEY_Control_L || event->keyval == GDK_KEY_Control_R) &&
-        this->spacingSide == Side::Above) {
+bool VerticalToolHandler::onKeyReleaseEvent(const KeyEvent& event) {
+    if ((event.keyval == GDK_KEY_Control_L || event.keyval == GDK_KEY_Control_R) && this->spacingSide == Side::Above) {
         this->adoptElements(Side::Below);
         this->viewPool->dispatch(xoj::view::VerticalToolView::SWITCH_DIRECTION_REQUEST);
         return true;
@@ -93,11 +90,21 @@ bool VerticalToolHandler::onKeyReleaseEvent(GdkEventKey* event) {
     return false;
 }
 
-auto VerticalToolHandler::getElements() const -> const std::vector<Element*>& { return this->elements; }
+auto VerticalToolHandler::refElements() const -> std::vector<Element*> {
+    auto result = std::vector<Element*>(this->elements.size());
+    std::transform(this->elements.begin(), this->elements.end(), result.begin(), [](auto const& e) { return e.get(); });
+    return result;
+}
+
+void VerticalToolHandler::forEachElement(std::function<void(const Element*)> f) const {
+    for (auto const& e: this->elements) {
+        f(e.get());
+    }
+}
 
 auto VerticalToolHandler::computeElementsBoundingBox() const -> Range {
     Range rg;
-    for (Element* e: this->elements) {
+    for (auto const& e: this->elements) {
         rg = rg.unite(Range(e->boundingRect()));
     }
     return rg;
@@ -113,13 +120,12 @@ auto VerticalToolHandler::finalize() -> std::unique_ptr<MoveUndoAction> {
     }
 
     const double dY = this->endY - this->startY;
-    auto undo =
-            std::make_unique<MoveUndoAction>(this->layer, this->page, &this->elements, 0, dY, this->layer, this->page);
+    auto undo = std::make_unique<MoveUndoAction>(this->layer, this->page, this->refElements(), 0, dY, this->layer,
+                                                 this->page);
 
-    for (Element* e: this->elements) {
+    for (auto&& e: this->elements) {
         e->move(0, dY);
-
-        this->layer->addElement(e);
+        this->layer->addElement(std::move(e));
     }
     this->elements.clear();
 
