@@ -149,7 +149,7 @@ auto Layout::getVisibleRect() -> Rectangle<double> {
  * adds the addend to base if the predicate is true
  */
 
-constexpr auto sumIf = [](auto base, auto addend, bool predicate) {
+[[maybe_unused]] constexpr auto sumIf = [](auto base, auto addend, bool predicate) {
     if constexpr (std::is_signed_v<decltype(base)> || std::is_signed_v<decltype(addend)>) {
         using RT = std::make_signed_t<decltype(base + addend)>;
         if (predicate) {
@@ -184,13 +184,24 @@ void Layout::recalculate_int() const {
     }
 
     // add space around the entire page area to accommodate older Wacom tablets with limited sense area.
-    auto const vPadding =
-            sumIf(XOURNAL_PADDING, settings->getAddVerticalSpaceAmount(), settings->getAddVerticalSpace());
-    auto const hPadding =
-            sumIf(XOURNAL_PADDING, settings->getAddHorizontalSpaceAmount(), settings->getAddHorizontalSpace());
+    auto vPadding = 2 * XOURNAL_PADDING;
+    if (settings->getUnlimitedScrolling()) {
+        vPadding += 2 * static_cast<int>(gtk_adjustment_get_page_size(scrollHandling->getVertical()));
+    } else if (settings->getAddVerticalSpace()) {
+        vPadding += settings->getAddVerticalSpaceAmountAbove();
+        vPadding += settings->getAddVerticalSpaceAmountBelow();
+    }
 
-    pc.minWidth = as_unsigned(2 * hPadding + as_signed_strict((pc.widthCols.size() - 1) * XOURNAL_PADDING_BETWEEN));
-    pc.minHeight = as_unsigned(2 * vPadding + as_signed_strict((pc.heightRows.size() - 1) * XOURNAL_PADDING_BETWEEN));
+    auto hPadding = 2 * XOURNAL_PADDING;
+    if (settings->getUnlimitedScrolling()) {
+        hPadding += 2 * static_cast<int>(gtk_adjustment_get_page_size(scrollHandling->getHorizontal()));
+    } else if (settings->getAddHorizontalSpace()) {
+        hPadding += settings->getAddHorizontalSpaceAmountLeft();
+        hPadding += settings->getAddHorizontalSpaceAmountRight();
+    }
+
+    pc.minWidth = as_unsigned(hPadding + as_signed_strict((pc.widthCols.size() - 1) * XOURNAL_PADDING_BETWEEN));
+    pc.minHeight = as_unsigned(vPadding + as_signed_strict((pc.heightRows.size() - 1) * XOURNAL_PADDING_BETWEEN));
 
     pc.minWidth = floor_cast<size_t>(std::accumulate(begin(pc.widthCols), end(pc.widthCols), double(pc.minWidth)));
     pc.minHeight = floor_cast<size_t>(std::accumulate(begin(pc.heightRows), end(pc.heightRows), double(pc.minHeight)));
@@ -222,10 +233,19 @@ void Layout::layoutPages(int width, int height) {
 
 
     // add space around the entire page area to accommodate older Wacom tablets with limited sense area.
-    auto const v_padding =
-            sumIf(XOURNAL_PADDING, settings->getAddVerticalSpaceAmount(), settings->getAddVerticalSpace());
-    auto const h_padding =
-            sumIf(XOURNAL_PADDING, settings->getAddHorizontalSpaceAmount(), settings->getAddHorizontalSpace());
+    auto v_padding = XOURNAL_PADDING;
+    if (settings->getUnlimitedScrolling()) {
+        v_padding += ceil_cast<int>(gtk_adjustment_get_page_size(scrollHandling->getVertical()));
+    } else if (settings->getAddVerticalSpace()) {
+        v_padding += settings->getAddVerticalSpaceAmountAbove();
+    }
+
+    auto h_padding = XOURNAL_PADDING;
+    if (settings->getUnlimitedScrolling()) {
+        h_padding += ceil_cast<int>(gtk_adjustment_get_page_size(scrollHandling->getHorizontal()));
+    } else if (settings->getAddHorizontalSpace()) {
+        h_padding += settings->getAddHorizontalSpaceAmountLeft();
+    }
 
     auto const centeringXBorder = (width - as_signed(pc.minWidth)) / 2;
     auto const centeringYBorder = (height - as_signed(pc.minHeight)) / 2;
@@ -311,13 +331,16 @@ auto Layout::getPaddingAbovePage(size_t pageIndex) const -> int {
     const Settings* settings = this->view->getControl()->getSettings();
 
     // User-configured padding above all pages.
-    auto const paddingAbove =
-            sumIf(XOURNAL_PADDING, settings->getAddVerticalSpaceAmount(), settings->getAddVerticalSpace());
+    auto paddingAbove = XOURNAL_PADDING;
+    if (settings->getUnlimitedScrolling()) {
+        paddingAbove += ceil_cast<int>(gtk_adjustment_get_page_size(scrollHandling->getVertical()));
+    } else if (settings->getAddVerticalSpace()) {
+        paddingAbove += settings->getAddVerticalSpaceAmountAbove();
+    }
 
-    // (x, y) coordinate pair gives grid indicies. This handles paired pages
-    // and different page layouts for us.
-    auto pageYLocation = this->mapper.at(pageIndex).row;
-    return strict_cast<int>(as_signed(pageYLocation) * XOURNAL_PADDING_BETWEEN + as_signed(paddingAbove));
+    // (x, y) coordinate pair gives grid indices. This handles paired pages and different page layouts for us.
+    auto pageYLocation = strict_cast<int>(this->mapper.at(pageIndex).row);
+    return pageYLocation * XOURNAL_PADDING_BETWEEN + paddingAbove;
 }
 
 
@@ -325,29 +348,28 @@ auto Layout::getPaddingLeftOfPage(size_t pageIndex) const -> int {
     bool isPairedPages = this->mapper.isPairedPages();
     const Settings* settings = this->view->getControl()->getSettings();
 
-    auto paddingBefore =
-            sumIf(XOURNAL_PADDING, settings->getAddHorizontalSpaceAmount(), settings->getAddHorizontalSpace());
+    auto paddingBefore = XOURNAL_PADDING;
+    if (settings->getUnlimitedScrolling()) {
+        paddingBefore += ceil_cast<int>(gtk_adjustment_get_page_size(scrollHandling->getHorizontal()));
+    } else if (settings->getAddHorizontalSpace()) {
+        paddingBefore += settings->getAddHorizontalSpaceAmountLeft();
+    }
 
-    auto const pageXLocation = as_signed(this->mapper.at(pageIndex).col);
+    auto const pageXLocation = strict_cast<int>(this->mapper.at(pageIndex).col);
 
-    // No page pairing or we haven't rendered enough pages in the row for
-    // page pairing to have an effect,
+    // No page pairing or we haven't rendered enough pages in the row for page pairing to have an effect,
     if (!isPairedPages) {
-        return strict_cast<int>(pageXLocation * XOURNAL_PADDING_BETWEEN + XOURNAL_PADDING_BETWEEN / 2 +
-                                as_signed(paddingBefore));
+        return pageXLocation * XOURNAL_PADDING_BETWEEN + XOURNAL_PADDING_BETWEEN / 2 + paddingBefore;
     } else {
-        auto columnPadding =
-                XOURNAL_PADDING_BETWEEN + strict_cast<int>(pageXLocation / 2) * (XOURNAL_PADDING_BETWEEN * 2);
+        auto columnPadding = XOURNAL_PADDING_BETWEEN + pageXLocation * XOURNAL_PADDING_BETWEEN;
         if (pageXLocation % 2 == 0) {
-            return strict_cast<int>(columnPadding - XOURNAL_ROOM_FOR_SHADOW + paddingBefore);
+            return columnPadding - XOURNAL_ROOM_FOR_SHADOW + paddingBefore;
         } else {
-            return strict_cast<int>(columnPadding + XOURNAL_ROOM_FOR_SHADOW + paddingBefore);
+            return columnPadding + XOURNAL_ROOM_FOR_SHADOW + paddingBefore;
         }
     }
 }
 
-void Layout::setLayoutSize(int width, int height) { this->scrollHandling->setLayoutSize(width, height); }
-
 void Layout::scrollRelative(double x, double y) {
     if (this->view->getControl()->getSettings()->isPresentationMode()) {
         return;
