@@ -22,8 +22,6 @@
 #include <glib.h>         // for gpointer, gboolean, gint
 #include <gtk/gtk.h>      // for GtkWidget, GtkCheckMenu...
 
-#include "control/layer/LayerCtrlListener.h"  // for LayerCtrlListener
-#include "model/Font.h"                       // for XojFont
 #include "util/Point.h"
 #include "util/raii/GObjectSPtr.h"
 
@@ -46,15 +44,15 @@ class Menubar;
 
 typedef std::array<xoj::util::WidgetSPtr, TOOLBAR_DEFINITIONS_LEN> ToolbarWidgetArray;
 
-class MainWindow: public GladeGui, public LayerCtrlListener {
+class MainWindow: public GladeGui {
 public:
     MainWindow(GladeSearchpath* gladeSearchPath, Control* control, GtkApplication* parent);
     ~MainWindow() override;
 
-    // LayerCtrlListener
+    void populate(GladeSearchpath* gladeSearchPath);
+
 public:
-    void rebuildLayerMenu() override;
-    void layerVisibilityChanged() override;
+    GMenuModel* getMenuModel() const;
 
     void show(GtkWindow* parent) override;
 
@@ -62,23 +60,25 @@ public:
     void toolbarSelected(ToolbarData* d);
     ToolbarData* getSelectedToolbar() const;
 
-    /**
-     * These methods are only used internally and for toolbar configuration
-     */
-    ToolbarData* clearToolbar();
+private:
+    const ToolbarData* clearToolbar();
     void loadToolbar(ToolbarData* d);
 
+public:
+    /**
+     * reloadToolbars reloads the currently selected toolbar
+     *
+     * This is especially useful when a change in the SettingsDialog should be reflected right away
+     */
+    void reloadToolbars();
 
     void updatePageNumbers(size_t page, size_t pagecount, size_t pdfpage);
 
-    void setFontButtonFont(const XojFont& font);
-    XojFont getFontButtonFont() const;
-
-    void saveSidebarSize();
-
     void setMaximized(bool maximized);
     bool isMaximized() const;
 
+    void setFullscreen(bool enabled) const;
+
     bool isDarkTheme() const;
 
     XournalView* getXournal() const;
@@ -97,15 +97,10 @@ public:
     void setUndoDescription(const std::string& description);
     void setRedoDescription(const std::string& description);
 
-    SpinPageAdapter* getSpinPageNo() const;
     ToolbarModel* getToolbarModel() const;
     ToolMenuHandler* getToolMenuHandler() const;
 
-    void disableAudioPlaybackButtons();
-    void enableAudioPlaybackButtons();
-    void setAudioPlaybackPaused(bool paused);
-
-    void setControlTmpDisabled(bool disabled);
+    void setDynamicallyGeneratedSubmenuDisabled(bool disabled);
 
     void updateToolbarMenu();
     void updateColorscheme();
@@ -123,7 +118,7 @@ public:
      *
      * @see Util::toWidgetCoords()
      */
-    utl::Point<double> getNegativeXournalWidgetPos() const;
+    xoj::util::Point<double> getNegativeXournalWidgetPos() const;
 
     /**
      * Disable kinetic scrolling if there is a touchscreen device that was manually mapped to another enabled input
@@ -132,34 +127,20 @@ public:
     void setGtkTouchscreenScrollingForDeviceMapping();
     void setGtkTouchscreenScrollingEnabled(bool enabled);
 
-    void rebindMenubarAccelerators();
+    /// Infer the window's DPI from available monitor info and use it to set the default zoom value.
+    void setDPI() const;
 
 private:
     void initXournalWidget();
 
-    /**
-     * Allow to hide menubar, but only if global menu is not enabled
-     */
-    void initHideMenu();
-    static void toggleMenuBar(MainWindow* win);
-
     void createToolbar();
-    static void rebindAcceleratorsMenuItem(GtkWidget* widget, gpointer user_data);
-    static void rebindAcceleratorsSubMenu(GtkWidget* widget, gpointer user_data);
-    static gboolean isKeyForClosure(GtkAccelKey* key, GClosure* closure, gpointer data);
-    static gboolean invokeMenu(GtkWidget* widget);
-
-    static void buttonCloseSidebarClicked(GtkButton* button, MainWindow* win);
 
     /**
-     * Sidebar show / hidden
+     * Update the position of the separator in the paned container, adjusting it to the saved sidebar width.
+     * @param contentWidth should be the width of the paned container. The caller should retrieve the width
+     * of the container before any modifications to it, as that will reset its allocation.
      */
-    static void viewShowSidebar(GtkCheckMenuItem* checkmenuitem, MainWindow* win);
-
-    /**
-     * Toolbar show / hidden
-     */
-    static void viewShowToolbar(GtkCheckMenuItem* checkmenuitem, MainWindow* win);
+    void updatePanedPosition(int contentWidth);
 
     /**
      * Window close Button is pressed
@@ -206,8 +187,6 @@ private:
 
     ToolbarWidgetArray toolbarWidgets;
 
-    GtkAccelGroup* globalAccelGroup;
-
     bool sidebarVisible = true;
 
     xoj::util::WidgetSPtr boxContainerWidget;
