@@ -28,7 +28,7 @@ struct ToolItemDragDropData {
     ToolItemType type;
     int id;
     AbstractToolItem* item;
-    const NamedColor* namedColor;
+    size_t paletteColorIndex;
 };
 
 class ToolitemDragDrop {
@@ -39,12 +39,11 @@ private:
 public:
     static void attachMetadata(GtkWidget* w, int id, AbstractToolItem* ait);
     static void attachMetadata(GtkWidget* w, int id, ToolItemType type);
-    static void attachMetadataColor(GtkWidget* w, int id, const NamedColor* namedColor, AbstractToolItem* item);
+    static void attachMetadataColor(GtkWidget* w, int id, size_t paletteColorIndex, AbstractToolItem* item);
 
 public:
     static std::unique_ptr<ToolItemDragDropData> ToolItemDragDropData_new(AbstractToolItem* item);
     static bool checkToolItemDragDropData(ToolItemDragDropData const* d);
-    static bool isToolItemEnabled(ToolItemDragDropData* d);
 
     static ToolItemDragDropData* metadataGetMetadata(GtkWidget* w);
 
@@ -52,13 +51,6 @@ public:
      * Returns: (transfer floating)
      */
     static GtkWidget* getIcon(ToolItemDragDropData* data);
-    /**
-     * @brief Get Pixbuf for the respective tool contained in the DragDrop data
-     *
-     * @param data DragDropData relevant during toolbar customization
-     * @return GdkPixbuf* of the dragged tool
-     */
-    static GdkPixbuf* getPixbuf(ToolItemDragDropData* data);
 
 public:
     static void removeFromToolbarForMove(GtkWidget* widget);
