@@ -42,4 +42,6 @@ constexpr static char CFG_LINE_WIDTH[] = "lw";
 constexpr static char CFG_MARGIN[] = "m1";
 constexpr static char CFG_ROUND_MARGIN[] = "rm";
 constexpr static char CFG_RASTER[] = "r1";
+constexpr static char CFG_BOLD_LINE_INTERVAL[] = "bli";
+constexpr static char CFG_BOLD_LINE_WIDTH[] = "blw";
 }  // namespace background_config_strings
