@@ -1,10 +1,10 @@
 #include "Document.h"
 
-#include <array>
 #include <codecvt>  // for codecvt_utf8_utf16
 #include <cstddef>
 #include <ctime>  // for size_t, localtime, strf...
 #include <iomanip>
+#include <memory>
 #include <sstream>
 #include <string>  // for string
 #include <string_view>
@@ -18,6 +18,7 @@
 #include "model/PageType.h"                   // for PageType
 #include "pdf/base/XojPdfAction.h"            // for XojPdfAction
 #include "pdf/base/XojPdfBookmarkIterator.h"  // for XojPdfBookmarkIterator
+#include "util/Assert.h"                      // for xoj_assert
 #include "util/PathUtil.h"                    // for clearExtensions
 #include "util/PlaceholderString.h"           // for PlaceholderString
 #include "util/SaveNameUtils.h"               // for parseFilename
@@ -25,6 +26,8 @@
 #include "util/glib_casts.h"                  // for wrap_v
 #include "util/i18n.h"                        // for FS, _F
 #include "util/raii/GObjectSPtr.h"            // for GObjectSPtr
+#include "util/safe_casts.h"                  // for as_signed
+#include "util/utf8_view.h"                   // for utf8
 
 #include "LinkDestination.h"  // for XojLinkDest, DOCUMENT_L...
 #include "XojPage.h"          // for XojPage
@@ -124,7 +127,7 @@ auto Document::getFilepath() const -> fs::path { return filepath; }
 
 auto Document::getPdfFilepath() const -> fs::path { return pdfFilepath; }
 
-auto Document::createSaveFolder(fs::path lastSavePath) -> fs::path {
+auto Document::createSaveFoldername(const fs::path& lastSavePath) const -> fs::path {
     if (!filepath.empty()) {
         return filepath.parent_path();
     }
@@ -147,25 +150,25 @@ auto Document::createSaveFolder(fs::path lastSavePath) -> fs::path {
  * @param formatStr The input format string to preprocess.
  * @return The processed format string with standardized specifiers.
  */
-static std::string preprocessFormatString(std::string formatStr) {
-    auto replace = [&formatStr](std::string_view pattern, std::string_view replacement) {
+static std::u8string preprocessFormatString(std::u8string formatStr) {
+    auto replace = [&formatStr](std::u8string_view pattern, std::u8string_view replacement) {
         for (size_t pos = formatStr.find(pattern); pos != std::string::npos;
              pos = formatStr.find(pattern, pos + replacement.length())) {
             formatStr.replace(pos, pattern.length(), replacement);
         }
     };
 
-    replace("%F", "%Y-%m-%d");
-    replace("%T", "%H-%M-%S");
-    replace("%V", "%U");
+    replace(u8"%F", u8"%Y-%m-%d");
+    replace(u8"%T", u8"%H-%M-%S");
+    replace(u8"%V", u8"%U");
 
     return formatStr;
 }
 
-auto Document::createSaveFilename(DocumentType type, const std::string& defaultSaveName,
-                                  const std::string& defaultPdfName) -> fs::path {
+auto Document::createSaveFilename(DocumentType type, std::u8string_view defaultSaveName,
+                                  std::u8string_view defaultPdfName) const -> fs::path {
     constexpr static std::wstring_view forbiddenChars = {L"\\/:*?\"<>|"};
-    std::string wildcardString;
+    std::u8string wildcardString;
     if (type != Document::PDF) {
         if (!filepath.empty()) {
             // This can be any extension
@@ -179,16 +182,17 @@ auto Document::createSaveFilename(DocumentType type, const std::string& defaultS
             return p;
         }
     } else if (!pdfFilepath.empty()) {
-        wildcardString = SaveNameUtils::parseFilenameFromWildcardString(defaultPdfName, this->pdfFilepath.filename());
+        wildcardString = SaveNameUtils::parseFilenameFromWildcardString(defaultPdfName, this->pdfFilepath.filename(),
+                                                                        this->filepath.filename());
     } else if (!filepath.empty()) {
-        wildcardString = SaveNameUtils::parseFilenameFromWildcardString(defaultPdfName, this->filepath.filename());
+        wildcardString = SaveNameUtils::parseFilenameFromWildcardString(defaultPdfName, this->filepath.filename(),
+                                                                        this->filepath.filename());
     }
 
-    auto format_str = wildcardString.empty() ? defaultSaveName : wildcardString;
+    auto format_str = preprocessFormatString(wildcardString.empty() ? defaultSaveName.data() : wildcardString);
 
-    format_str = preprocessFormatString(format_str);
     std::wstring_convert<std::codecvt_utf8<wchar_t>> converter;
-    auto format = converter.from_bytes(format_str);
+    auto format = converter.from_bytes(char_cast(format_str).data());
 
     // Todo (cpp20): use <format>
     std::wostringstream ss;
@@ -203,7 +207,7 @@ auto Document::createSaveFilename(DocumentType type, const std::string& defaultS
     }
 
     auto fn2 = converter.to_bytes(filename);
-    auto p = fs::u8path(fn2);
+    auto p = fs::path(xoj::util::utf8(fn2));
 
     Util::clearExtensions(p);
     return p;
@@ -234,13 +238,12 @@ auto Document::getEvMetadataFilename() const -> fs::path {
 
 auto Document::isAttachPdf() const -> bool { return this->attachPdf; }
 
-auto Document::findPdfPage(size_t pdfPage) -> size_t {
+auto Document::findPdfPage(size_t pdfPage) const -> size_t {
     // Create a page index if not already indexed.
-    if (!this->pageIndex)
-        indexPdfPages();
+    xoj_assert(this->pageIndex);
     auto pos = this->pageIndex->find(pdfPage);
     if (pos == this->pageIndex->end()) {
-        return -1;
+        return npos;
     } else {
         return pos->second;
     }
@@ -335,6 +338,7 @@ auto Document::fillPageLabels(GtkTreeModel* treeModel, GtkTreePath* path, GtkTre
 }
 
 void Document::updateIndexPageNumbers() {
+    indexPdfPages();
     if (this->contentsModel) {
         gtk_tree_model_foreach(this->contentsModel.get(), xoj::util::wrap_v<fillPageLabels>, this);
     }
@@ -345,14 +349,14 @@ void Document::setPdfAttributes(const fs::path& filename, bool attachToDocument)
     this->attachPdf = attachToDocument;
 }
 
-auto Document::readPdf(const fs::path& filename, bool initPages, bool attachToDocument, gpointer data, gsize length)
-        -> bool {
+auto Document::readPdf(const fs::path& filename, bool initPages, bool attachToDocument,
+                       std::unique_ptr<std::string> data) -> bool {
     GError* popplerError = nullptr;
 
     lock();
 
     if (data != nullptr) {
-        if (!pdfDocument.load(data, length, password, &popplerError)) {
+        if (!pdfDocument.load(std::move(data), password, &popplerError)) {
             lastError = FS(_F("Document not loaded! ({1}), {2}") % filename.u8string() % popplerError->message);
             g_error_free(popplerError);
             unlock();
@@ -414,7 +418,7 @@ auto Document::getPageHeight(PageRef p) -> double { return p->getHeight(); }
 auto Document::getLastErrorMsg() const -> std::string { return lastError; }
 
 void Document::deletePage(size_t pNr) {
-    auto it = this->pages.begin() + pNr;
+    auto it = this->pages.begin() + as_signed(pNr);
     this->pages.erase(it);
 
     // Reset the page index
@@ -423,7 +427,7 @@ void Document::deletePage(size_t pNr) {
 }
 
 void Document::insertPage(const PageRef& p, size_t position) {
-    this->pages.insert(this->pages.begin() + position, p);
+    this->pages.insert(this->pages.begin() + as_signed(position), p);
 
     // Reset the page index
     this->pageIndex.reset();
@@ -476,6 +480,7 @@ auto Document::operator=(const Document& doc) -> Document& {
     this->filepath = doc.filepath;
     this->pages = doc.pages;
     this->attachPdf = doc.attachPdf;
+    this->pathStorageMode = doc.pathStorageMode;
 
     indexPdfPages();
     buildContentsModel();
