@@ -33,8 +33,8 @@ public:
 
     virtual void rectChanged(xoj::util::Rectangle<double>& rect) {}
     virtual void rangeChanged(Range& range) {}
-    virtual void elementChanged(Element* elem) {}
-    virtual void elementsChanged(const std::vector<Element*>& elements, const Range& range) {}
+    virtual void elementChanged(const Element* elem) {}
+    virtual void elementsChanged(const std::vector<const Element*>& elements, const Range& range) {}
     virtual void pageChanged() {}
 
 private:
