@@ -19,6 +19,8 @@
 #include "model/PageType.h"                 // for PageType
 #include "model/XojPage.h"                  // for XojPage
 #include "pdf/base/XojPdfPage.h"            // for XojPdfPageSPtr, XojPdfPage
+#include "util/Assert.h"                    // for xoj_assert
+#include "util/StringUtils.h"               // for char_cast
 #include "util/Util.h"                      // for npos
 #include "util/i18n.h"                      // for _
 #include "util/serdesstream.h"              // for serdes_stream
@@ -27,7 +29,7 @@
 #include "config.h"      // for PROJECT_STRING
 #include "filesystem.h"  // for path
 
-XojCairoPdfExport::XojCairoPdfExport(Document* doc, ProgressListener* progressListener):
+XojCairoPdfExport::XojCairoPdfExport(const Document* doc, ProgressListener* progressListener):
         doc(doc), progressListener(progressListener) {}
 
 XojCairoPdfExport::~XojCairoPdfExport() {
@@ -44,24 +46,28 @@ void XojCairoPdfExport::setExportBackground(ExportBackgroundType exportBackgroun
 }
 
 auto XojCairoPdfExport::startPdf(const fs::path& file, bool exportOutline) -> bool {
-    this->surface = cairo_pdf_surface_create(file.u8string().c_str(), 0, 0);
+    this->surface = cairo_pdf_surface_create(char_cast(file.u8string().c_str()), 0, 0);
     this->cr = cairo_create(surface);
 
 #if CAIRO_VERSION >= CAIRO_VERSION_ENCODE(1, 16, 0)
-    cairo_pdf_surface_set_metadata(surface, CAIRO_PDF_METADATA_TITLE, doc->getFilepath().filename().u8string().c_str());
+    cairo_pdf_surface_set_metadata(surface, CAIRO_PDF_METADATA_TITLE,
+                                   char_cast(doc->getFilepath().filename().u8string().c_str()));
     cairo_pdf_surface_set_metadata(surface, CAIRO_PDF_METADATA_CREATOR, PROJECT_STRING);
     if (exportOutline) {
         this->populatePdfOutline();
     }
 #endif
+    configureCairoFontOptions();
 
+    return cairo_surface_status(this->surface) == CAIRO_STATUS_SUCCESS;
+}
+
+void XojCairoPdfExport::configureCairoFontOptions() {
     // Turn on font hint metrics, for consistency with text display in the app
     cairo_font_options_t* fontOptions = cairo_font_options_create();
     cairo_font_options_set_hint_metrics(fontOptions, CAIRO_HINT_METRICS_ON);
     cairo_set_font_options(cr, fontOptions);
     cairo_font_options_destroy(fontOptions);
-
-    return cairo_surface_status(this->surface) == CAIRO_STATUS_SUCCESS;
 }
 
 #if CAIRO_VERSION >= CAIRO_VERSION_ENCODE(1, 16, 0)
@@ -130,7 +136,7 @@ bool XojCairoPdfExport::endPdf() {
     return success;
 }
 
-void XojCairoPdfExport::exportPage(size_t page) {
+void XojCairoPdfExport::exportPage(size_t page, bool exportPdfBackground) {
     PageRef p = doc->getPage(page);
 
     cairo_pdf_surface_set_size(this->surface, p->getWidth(), p->getHeight());
@@ -140,20 +146,24 @@ void XojCairoPdfExport::exportPage(size_t page) {
     cairo_save(this->cr);
 
     // For a better pdf quality, we use a dedicated pdf rendering
-    if (p->getBackgroundType().isPdfPage() && (exportBackground != EXPORT_BACKGROUND_NONE)) {
+    if (exportPdfBackground && p->getBackgroundType().isPdfPage() && (exportBackground != EXPORT_BACKGROUND_NONE)) {
         auto pgNo = p->getPdfPageNr();
         XojPdfPageSPtr popplerPage = doc->getPdfPage(pgNo);
 
         popplerPage->renderForPrinting(cr);
     }
 
+    xoj::view::BackgroundFlags flags;
+    flags.showPDF = xoj::view::HIDE_PDF_BACKGROUND;  // Already exported (if any)
+    flags.showImage = exportBackground == EXPORT_BACKGROUND_NONE ? xoj::view::HIDE_IMAGE_BACKGROUND :
+                                                                   xoj::view::SHOW_IMAGE_BACKGROUND;
+    flags.showRuling = exportBackground <= EXPORT_BACKGROUND_UNRULED ? xoj::view::HIDE_RULING_BACKGROUND :
+                                                                       xoj::view::SHOW_RULING_BACKGROUND;
+
     if (layerRange) {
-        view.drawLayersOfPage(*layerRange, p, this->cr, true /* dont render eraseable */,
-                              true /* don't rerender the pdf background */, exportBackground == EXPORT_BACKGROUND_NONE,
-                              exportBackground <= EXPORT_BACKGROUND_UNRULED);
+        view.drawLayersOfPage(*layerRange, p, this->cr, true /* dont render eraseable */, flags);
     } else {
-        view.drawPage(p, this->cr, true /* dont render eraseable */, true /* don't rerender the pdf background */,
-                      exportBackground == EXPORT_BACKGROUND_NONE, exportBackground <= EXPORT_BACKGROUND_UNRULED);
+        view.drawPage(p, this->cr, true /* dont render eraseable */, flags);
     }
 
     // next page
@@ -167,20 +177,20 @@ void XojCairoPdfExport::exportPageLayers(size_t page) {
 
     // We keep a copy of the layers initial Visible state
     std::map<Layer*, bool> initialVisibility;
-    for (const auto& layer: *p->getLayers()) {
+    for (const auto& layer: p->getLayers()) {
         initialVisibility[layer] = layer->isVisible();
         layer->setVisible(false);
     }
 
     // We draw as many pages as there are layers. The first page has
     // only Layer 1 visible, the last has all layers visible.
-    for (const auto& layer: *p->getLayers()) {
+    for (const auto& layer: p->getLayers()) {
         layer->setVisible(true);
         exportPage(page);
     }
 
     // We restore the initial visibilities
-    for (const auto& layer: *p->getLayers()) layer->setVisible(initialVisibility[layer]);
+    for (const auto& layer: p->getLayers()) layer->setVisible(initialVisibility[layer]);
 }
 
 auto XojCairoPdfExport::createPdf(fs::path const& file, const PageRangeVector& range, bool progressiveMode) -> bool {
@@ -201,7 +211,10 @@ auto XojCairoPdfExport::createPdf(fs::path const& file, const PageRangeVector& r
     }
 
     size_t count = 0;
-    for (const auto& e: range) { count += e.last - e.first + 1; }
+    for (const auto& e: range) {
+        xoj_assert(e.last >= e.first);  // Ok, when the PageRangeVector was the result of parsing
+        count += e.last - e.first + 1;  // Not accurate, if e.last is > doc->getPageCount()
+    }
 
     if (this->progressListener) {
         this->progressListener->setMaximumState(count);
@@ -209,7 +222,7 @@ auto XojCairoPdfExport::createPdf(fs::path const& file, const PageRangeVector& r
 
     size_t c = 0;
     for (const auto& e: range) {
-        auto max = std::min(e.last, doc->getPageCount());
+        auto max = std::min(e.last, doc->getPageCount());  // Should be e.last for parsed PageRangeVector
         for (size_t i = e.first; i <= max; i++) {
             if (progressiveMode) {
                 exportPageLayers(i);
@@ -218,7 +231,7 @@ auto XojCairoPdfExport::createPdf(fs::path const& file, const PageRangeVector& r
             }
 
             if (this->progressListener) {
-                this->progressListener->setCurrentState(c++);
+                this->progressListener->setCurrentState(++c);
             }
         }
     }
@@ -252,7 +265,7 @@ auto XojCairoPdfExport::createPdf(fs::path const& file, bool progressiveMode) ->
         }
 
         if (this->progressListener) {
-            this->progressListener->setCurrentState(i);
+            this->progressListener->setCurrentState(i + 1);
         }
     }
 
