@@ -33,7 +33,7 @@ public:
 public:
     virtual bool save(fs::path const& file, GError** error) const = 0;
     virtual bool load(fs::path const& file, std::string password, GError** error) = 0;
-    virtual bool load(gpointer data, gsize length, std::string password, GError** error) = 0;
+    virtual bool load(std::unique_ptr<std::string> data, std::string password, GError** error) = 0;
     virtual bool isLoaded() const = 0;
     virtual void reset() = 0;
 
