@@ -28,14 +28,15 @@ public:
     DeleteUndoAction(const PageRef& page, bool eraser);
 
 public:
-    bool undo(Control*) override;
-    bool redo(Control*) override;
+    bool undo(Control* control) override;
+    bool redo(Control* control) override;
 
-    void addElement(Layer* layer, Element* e, Element::Index pos);
+    void addElement(Layer* layer, ElementPtr e, Element::Index pos);
 
     std::string getText() override;
 
 private:
+    // Todo (performance): replace by flat_multi_set / sorted_vector
     std::multiset<PageLayerPosEntry<Element>> elements{};
     bool eraser = true;
 };
