@@ -14,6 +14,7 @@
 #include <string>  // for string
 #include <vector>  // for vector
 
+#include "model/Element.h"
 #include "model/PageRef.h"  // for PageRef
 
 #include "UndoAction.h"  // for UndoAction
@@ -22,9 +23,11 @@ class Element;
 class Layer;
 class Control;
 
+using ElementPtr = std::unique_ptr<Element>;
+
 class InsertUndoAction: public UndoAction {
 public:
-    InsertUndoAction(const PageRef& page, Layer* layer, Element* element);
+    InsertUndoAction(const PageRef& page, Layer* layer, const Element* element);
     ~InsertUndoAction() override;
 
 public:
@@ -35,12 +38,13 @@ public:
 
 private:
     Layer* layer;
-    Element* element;
+    const Element* element;
+    ElementPtr elementOwn;
 };
 
 class InsertsUndoAction: public UndoAction {
 public:
-    InsertsUndoAction(const PageRef& page, Layer* layer, std::vector<Element*> elements);
+    InsertsUndoAction(const PageRef& page, Layer* layer, std::vector<const Element*> elements);
     ~InsertsUndoAction() override;
 
 public:
@@ -51,5 +55,6 @@ public:
 
 private:
     Layer* layer;
-    std::vector<Element*> elements;
+    std::vector<const Element*> elements;
+    std::vector<ElementPtr> elementsOwn;
 };
