@@ -1,7 +1,6 @@
 #include "ElementContainerView.h"
 
 #include <memory>  // for unique_ptr
-#include <vector>  // for vector
 
 #include "model/ElementContainer.h"  // for ElementContainer
 
@@ -14,8 +13,8 @@ using namespace xoj::view;
 ElementContainerView::ElementContainerView(const ElementContainer* container): container(container) {}
 
 void ElementContainerView::draw(const Context& ctx) const {
-    for (Element* e: container->getElements()) {
+    container->forEachElement([&ctx](const Element* e) {
         auto elementView = ElementView::createFromElement(e);
         elementView->draw(ctx);
-    }
+    });
 }
