@@ -6,34 +6,45 @@
 
 #include "util/PlaceholderString.h"                 // for PlaceholderString
 #include "util/i18n.h"                              // for FORMAT_STR, FS
+#include "util/safe_casts.h"                        // for as_signed
 #include "util/serializing/InputStreamException.h"  // for InputStreamException
 #include "util/serializing/Serializable.h"          // for XML_VERSION_STR
 
-template int ObjectInputStream::readType<int>();
+static size_t getSize(std::stringstream& s) {
+    auto pos = s.tellg();
+    s.seekg(0, std::ios::end);
+    auto size = s.tellg() - pos;
+    s.seekg(pos);
+    return as_unsigned(size);
+}
+
+template size_t ObjectInputStream::readType<size_t>();
 
 // This function requires that T is read from its binary representation to work (e.g. integer type)
 template <typename T>
 T ObjectInputStream::readType() {
-    if (istream.str().size() < sizeof(T)) {
+    if (getSize(istream) < sizeof(T)) {
         std::ostringstream oss;
-        oss << "End reached: trying to read " << sizeof(T) << " bytes while only " << istream.str().size()
+        oss << "End reached: trying to read " << sizeof(T) << " bytes while only " << getSize(istream)
             << " bytes available";
         throw InputStreamException(oss.str(), __FILE__, __LINE__);
     }
     T output;
 
-    istream.read((char*)&output, sizeof(T));
+    istream.read((char*)&output, as_signed(sizeof(T)));
 
     return output;
 }
 
-size_t ObjectInputStream::pos() { return istream.tellg(); }
+size_t ObjectInputStream::pos() { return static_cast<size_t>(istream.tellg()); }
 
-auto ObjectInputStream::read(const char* data, int data_len) -> bool {
-    istream.clear();
-    len = (size_t)data_len;
-    std::string dataStr = std::string(data, len);
-    istream.str(dataStr);
+auto ObjectInputStream::read(const char* data, size_t data_len) -> bool {
+    return read(std::stringstream(std::string(data, data_len)), data_len);
+}
+
+auto ObjectInputStream::read(std::stringstream stream, size_t length) -> bool {
+    this->istream = std::move(stream);
+    this->len = length;
 
     try {
         std::string version = readString();
@@ -79,6 +90,11 @@ auto ObjectInputStream::readInt() -> int {
     return readType<int>();
 }
 
+auto ObjectInputStream::readUInt() -> uint32_t {
+    checkType('u');
+    return readType<uint32_t>();
+}
+
 auto ObjectInputStream::readDouble() -> double {
     checkType('d');
     return readType<double>();
@@ -92,16 +108,19 @@ auto ObjectInputStream::readSizeT() -> size_t {
 auto ObjectInputStream::readString() -> std::string {
     checkType('s');
 
-    size_t lenString = (size_t)readType<int>();
+    size_t lenString = readType<size_t>();
 
-    if (istream.str().size() < len) {
-        throw InputStreamException("End reached, but try to read an string", __FILE__, __LINE__);
+    if (getSize(istream) < lenString) {
+        std::ostringstream oss;
+        oss << "End reached: trying to read a string of " << lenString << " bytes while only " << istream.str().size()
+            << " bytes are available";
+        throw InputStreamException(oss.str(), __FILE__, __LINE__);
     }
 
     std::string output;
     output.resize(lenString);
 
-    istream.read(&output[0], (long)lenString);
+    istream.read(&output[0], as_signed(lenString));
 
     return output;
 }
@@ -109,33 +128,36 @@ auto ObjectInputStream::readString() -> std::string {
 auto ObjectInputStream::readImage() -> std::string {
     checkType('m');
 
-    if (istream.str().size() < sizeof(size_t)) {
+    if (getSize(istream) < sizeof(size_t)) {
         throw InputStreamException("End reached, but try to read an image's data's length", __FILE__, __LINE__);
     }
 
-    const size_t len = readType<size_t>();
-    if (istream.str().size() < len) {
-        throw InputStreamException("End reached, but try to read an image", __FILE__, __LINE__);
+    const size_t lenString = readType<size_t>();
+    if (getSize(istream) < lenString) {
+        std::ostringstream oss;
+        oss << "End reached: trying to read an image of " << lenString << " bytes while only " << istream.str().size()
+            << " bytes are available";
+        throw InputStreamException(oss.str(), __FILE__, __LINE__);
     }
     std::string data;
-    data.resize(len);
-    istream.read(data.data(), static_cast<int>(len));
+    data.resize(lenString);
+    istream.read(data.data(), as_signed(lenString));
 
     return data;
 }
 
 void ObjectInputStream::checkType(char type) {
-    if (istream.str().size() < 2) {
-        throw InputStreamException(FS(FORMAT_STR("End reached, but try to read {1}, index {2} of {3}") % getType(type) %
-                                      (uint32_t)pos() % (uint32_t)len),
-                                   __FILE__, __LINE__);
+    if (getSize(istream) < 2) {
+        throw InputStreamException(
+                FS(FORMAT_STR("End reached, but try to read {1}, index {2} of {3}") % getType(type) % pos() % len),
+                __FILE__, __LINE__);
     }
     char t = 0, underscore = 0;
     istream >> underscore >> t;
 
     if (underscore != '_') {
         throw InputStreamException(FS(FORMAT_STR("Expected type signature of {1}, index {2} of {3}, but read '{4}'") %
-                                      getType(type) % ((uint32_t)pos() + 1) % (uint32_t)len % underscore),
+                                      getType(type) % (pos() + 1) % len % underscore),
                                    __FILE__, __LINE__);
     }
 
@@ -153,8 +175,12 @@ auto ObjectInputStream::getType(char type) -> std::string {
         ret = "Object end";
     } else if (type == 'i') {
         ret = "Number";
+    } else if (type == 'u') {
+        ret = "Unsigned Number";
     } else if (type == 'd') {
         ret = "Floating point";
+    } else if (type == 'l') {
+        ret = "Size";
     } else if (type == 's') {
         ret = "String";
     } else if (type == 'b') {
